package libconnect.lighthttp;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */



/**
 *
 * @author chris
 */
public class LightHTTPProxy implements Runnable {

    private String domain;
    private String cookie;
    private int port;

    private HashMap<String, byte[]> map = new HashMap<String, byte[]>();

    public LightHTTPProxy(String hotel, String cookie, int port){
        System.out.println(hotel);
        domain = (hotel.startsWith("www") ? hotel : ("www." + hotel));
        this.cookie = cookie;
        this.port = port;
    }

    public void setFile(String uri, byte[] data){
        map.put(uri, data);
    }

    public void Start(){
        new Thread(this, "LightHTTPProxy").start();
    }

    public void run(){
        try {
            ServerSocket serv = new ServerSocket(port);
            Socket sock;
            System.out.println("Listening on Port "+port);
            while((sock = serv.accept()) != null){
                new Proxy(sock);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    int inst = 0;

    class Proxy implements Runnable {

        Socket client, server;

        InputStream cin, sin;
        OutputStream cout, sout;

        public Proxy(Socket s) throws Exception{
            //System.out.println("Client Connected");
            client = s;
            server = new Socket(domain, 80);
            cout = client.getOutputStream();
            sout = server.getOutputStream();
            cin  = client.getInputStream();
            sin  = server.getInputStream();
            new Thread(this, "ProxyInstance-"+(inst++)).start();
        }

        public void run() {
            try{
                StringBuilder b = new StringBuilder();
                String uri = null;
                byte[] buff = new byte[1024];
                int i, len, r;
                boolean close;
                while(true){
                    boolean inMap = false;
                    b.delete(0, b.length());
                    len = 0;
                    ByteArrayOutputStream bout = new ByteArrayOutputStream();
                    while((i = cin.read()) > -1){
                        if(i == '\r') continue;
                        if(i == '\n'){
                            if(b.length() <= 0){
                                break;
                            }
                            if(b.indexOf("GET /") == 0 || b.indexOf("POST /") == 0) {
                                uri = b.substring(b.indexOf(" ")+1, b.lastIndexOf(" "));
                                System.out.println("Requested: "+uri);
                                if(map.containsKey(uri)) inMap = true;
                                else bout.write(b.toString().getBytes());
                            } else if(b.indexOf("Content-Length: ") == 0){
                                len = Integer.valueOf(b.substring(16));
                                if(!inMap) bout.write(b.toString().getBytes());
                            } else if(b.indexOf("Host: ") == 0){
                                if(!inMap){
                                    bout.write("Host: ".getBytes());
                                    bout.write(domain.getBytes());
                                }
                            } else if(b.indexOf("Accept-Encoding: ") == 0) {//Ignore GZIP
                                b.delete(0, b.length());
                                continue; // it was this line because it allowed another \r\n
                            } else if(b.indexOf("Cookie: ") == 0){
                                if(!inMap){
                                    bout.write("Cookie: ".getBytes());
                                    bout.write(cookie.getBytes());
                                    bout.write(';');
                                    bout.write(' ');
                                    bout.write(b.substring(b.indexOf(" ") + 1).getBytes());
                                }
                            } else if(!inMap) {
                                bout.write(b.toString().getBytes());
                            }
                            if(!inMap){
                                bout.write('\r');
                                bout.write('\n');
                            }
                            b.delete(0, b.length());
                        } else {
                            b.append((char) i);
                        }

                    }

                    if(inMap){
                        //System.out.println("Sending Map File");
                        cout.write(("HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nSet-Cookie: "+cookie+"\r\nConnection: keep-alive\r\nContent-Length: ").getBytes());
                        cout.write(String.valueOf(map.get(uri).length).getBytes());
                        cout.write("\r\n\r\n".getBytes());
                        cout.write(map.get(uri));
                    } else {
                        //System.out.println("Proxying Data");

                        bout.write('\r');
                        bout.write('\n');

                      //  ,,..S//ystem.out.write(bout.toByteArray());

                        sout.write(bout.toByteArray());

                        close = false;

                        if(len > 0){
                            r = len;
                            if(r > 1024) r = 1024;
                            while((i = cin.read(buff, 0, r)) > 0){
                                sout.write(buff, 0, i);
                                len -= i;
                                r = len;
                                if(r > 1024) r = 1024;
                            }
                        }

                        //System.out.println("Reading Server Response");

                        len = 0;
                        b.delete(0, b.length());
                        while((i = sin.read()) > -1){
                            if(i == '\r') continue;
                            if(i == '\n'){
                                if(b.length() <= 0){
                                    break;
                                }
                                if(b.indexOf("HTTP/1.0 ") == 0) {
                                    if(b.indexOf("200 OK") == -1){
                                        System.err.println(" ** Bad Response **");
                                    }
                                    cout.write(b.toString().getBytes());
                                } else if(b.indexOf("Content-Length: ") == 0){
                                    len = Integer.valueOf(b.substring(16));
                                    cout.write(b.toString().getBytes());
                                } else if(b.indexOf("Connection: close") == 0){
                                    close = true;
                                    cout.write(b.toString().getBytes());
                                } else {
                                    cout.write(b.toString().getBytes());
                                }
                                cout.write('\r');
                                cout.write('\n');

                                //System.out.println(b.toString());
                                b.delete(0, b.length());
                            } else {
                                b.append((char) i);
                            }
                        }

                        cout.write('\r');
                        cout.write('\n');

                        r = len;
                        if(r > 1024) r = 1024;

                        while((i = sin.read(buff, 0, r)) > 0){
                            cout.write(buff, 0, i);
                            len -= i;
                            r = len;
                            if(r > 1024) r = 1024;
                        }

                        if(close) break;

                    }

                }
            }catch(Exception ea){
                    //ea.printStackTrace();
            }
            try {
                client.close();
                server.close();
            } catch (IOException ex) {}

            System.out.println("Client Disconnected");
        }

    }

}
